@@ -111,7 +111,7 @@ struct NautilusFileDetails
 	eel_boolean_bit directory_count_failed        : 1;
 	eel_boolean_bit directory_count_is_up_to_date : 1;
 
-	NautilusRequestStatus deep_counts_status      : 2;
+	eel_boolean_bit deep_counts_status      : 2; /* NautilusRequestStatus */
 	/* no deep_counts_are_up_to_date field; since we expose
            intermediate values for this attribute, we do actually
            forget it rather than invalidating. */
@@ -125,6 +125,14 @@ struct NautilusFileDetails
 
 	eel_boolean_bit got_link_info                 : 1;
 	eel_boolean_bit link_info_is_up_to_date       : 1;
+
+	eel_boolean_bit is_thumbnailing               : 1;
+
+	eel_boolean_bit has_volume                    : 1;
+	eel_boolean_bit has_drive                     : 1;
+
+	/* TRUE if the file is open in a spatial window */
+	eel_boolean_bit has_open_window               : 1;
 };
 
 NautilusFile *nautilus_file_new_from_info                  (NautilusDirectory      *directory,
@@ -153,6 +161,10 @@ gboolean      nautilus_file_update_info                    (NautilusFile
 gboolean      nautilus_file_update_name                    (NautilusFile           *file,
 							    const char             *name);
 
+gboolean      nautilus_file_update_name_and_directory      (NautilusFile           *file,
+							    const char             *name,
+							    NautilusDirectory      *directory);
+
 /* Return true if the top lefts of files in this directory should be
  * fetched, according to the preference settings.
  */
@@ -161,13 +173,23 @@ gboolean      nautilus_file_should_get_top_left_text       (NautilusFile
 /* Mark specified attributes for this file out of date without canceling current
  * I/O or kicking off new I/O.
  */
-void          nautilus_file_invalidate_attributes_internal (NautilusFile           *file,
-							    GList                  *file_attributes);
-GList *       nautilus_file_get_all_attributes             (void);
-gboolean      nautilus_file_is_self_owned                  (NautilusFile           *file);
-void          nautilus_file_invalidate_count_and_mime_list (NautilusFile           *file);
-gboolean      nautilus_file_rename_in_progress             (NautilusFile           *file);
+void                   nautilus_file_invalidate_attributes_internal (NautilusFile           *file,
+								     NautilusFileAttributes  file_attributes);
+NautilusFileAttributes nautilus_file_get_all_attributes             (void);
+gboolean               nautilus_file_is_self_owned                  (NautilusFile           *file);
+void                   nautilus_file_invalidate_count_and_mime_list (NautilusFile           *file);
+gboolean               nautilus_file_rename_in_progress             (NautilusFile           *file);
+GnomeVFSFileInfo *     nautilus_file_peek_vfs_file_info             (NautilusFile           *file);
 
-GnomeVFSFileInfo *nautilus_file_peek_vfs_file_info         (NautilusFile           *file);
+
+/* Thumbnailing: */
+void          nautilus_file_set_is_thumbnailing            (NautilusFile           *file,
+							    gboolean                is_thumbnailing);
+
+/* Volumes: */
+void nautilus_file_set_drive  (NautilusFile   *file,
+			       GnomeVFSDrive  *drive);
+void nautilus_file_set_volume (NautilusFile   *file,
+			       GnomeVFSVolume *volume);
 
 #endif
