@@ -1095,22 +1095,22 @@ static void
 report_side_panel_failure_to_user (NautilusWindow *window, NautilusViewFrame *view_frame)
 {
 	char *message;
+	char *detail;
 	char *label;
 
 	label = nautilus_window_get_view_frame_label (view_frame);
 
         if (label == NULL) {
                 message = g_strdup
-                        (_("One of the side panels encountered an error and can't continue. "
-                           "Unfortunately I couldn't tell which one."));
+                        (_("One of the side panels encountered an error and can't continue."));
+		detail = _("Unfortunately I couldn't tell which one.");
         } else {
                 message = g_strdup_printf
-                        (_("The %s side panel encountered an error and can't continue. "
-                           "If this keeps happening, you might want to turn this panel off."),
-                         label);
+                        (_("The %s side panel encountered an error and can't continue."), label);
+                detail = _("If this keeps happening, you might want to turn this panel off.");
         }
 
-	eel_show_error_dialog (message, _("Side Panel Failed"), GTK_WINDOW (window));
+	eel_show_error_dialog (message, detail, _("Side Panel Failed"), GTK_WINDOW (window));
 
 	g_free (label);
 	g_free (message);
