@@ -63,9 +63,7 @@ enum {
 	LOAD_COMPLETE,
 	LOAD_PROGRESS_CHANGED,
 	LOAD_UNDERWAY,
-	OPEN_LOCATION_FORCE_NEW_WINDOW,
-	OPEN_LOCATION_IN_THIS_WINDOW,
-	OPEN_LOCATION_PREFER_EXISTING_WINDOW,
+	OPEN_LOCATION,
 	REPORT_LOCATION_CHANGE,
 	REPORT_REDIRECT,
 	TITLE_CHANGED,
@@ -95,10 +93,13 @@ struct NautilusViewFrameDetails {
 	BonoboEventSource *event_source;
 	BonoboControlFrame *control_frame;
         BonoboZoomableFrame *zoomable_frame;
-        
+
         /* The view CORBA object. */
         Nautilus_View view;
 
+	/* The positionable CORBA object. */
+	Nautilus_ScrollPositionable positionable;
+	
 	/* A container to connect our clients to. */
 	BonoboUIContainer *ui_container;
         NautilusUndoManager *undo_manager;
@@ -109,6 +110,16 @@ struct NautilusViewFrameDetails {
 
 	guint failed_idle_id;
 	guint socket_gone_idle_id;
+
+	/* zoom data */
+	float zoom_level;
+	float min_zoom_level;
+	float max_zoom_level;
+	gboolean has_min_zoom_level;
+	gboolean has_max_zoom_level;
+	GList *zoom_levels;
+
+	Nautilus_WindowType window_type;
 };
 
 static void nautilus_view_frame_init       (NautilusViewFrame      *view);
@@ -153,10 +164,14 @@ nautilus_view_frame_init (NautilusViewFrame *view)
 
 	view->details->idle_queue = nautilus_idle_queue_new ();
 
-	g_signal_connect_object (nautilus_signaller_get_current (), "history_list_changed",
-				 G_CALLBACK (send_history), view, G_CONNECT_SWAPPED);
-	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
-				 G_CALLBACK (send_history), view, G_CONNECT_SWAPPED);
+	g_signal_connect_object (nautilus_signaller_get_current (),
+				 "history_list_changed",
+				 G_CALLBACK (send_history),
+				 view, G_CONNECT_SWAPPED);
+	g_signal_connect_object (nautilus_icon_factory_get (),
+				 "icons_changed",
+				 G_CALLBACK (send_history),
+				 view, G_CONNECT_SWAPPED);
 }
 
 static void
@@ -178,6 +193,11 @@ destroy_view (NautilusViewFrame *view)
 	g_free (view->details->view_iid);
 	view->details->view_iid = NULL;
 
+	if (view->details->positionable != CORBA_OBJECT_NIL) {
+		bonobo_object_release_unref (view->details->positionable, NULL);
+		view->details->positionable = CORBA_OBJECT_NIL;
+	}
+	
 	CORBA_Object_release (view->details->view, NULL);
 	view->details->view = CORBA_OBJECT_NIL;
 
@@ -212,11 +232,11 @@ shut_down (NautilusViewFrame *view)
 	}
 
 	if (view->details->failed_idle_id != 0) {
-		gtk_idle_remove (view->details->failed_idle_id);
+		g_source_remove (view->details->failed_idle_id);
 		view->details->failed_idle_id = 0;
 	}
 	if (view->details->socket_gone_idle_id != 0) {
-		gtk_idle_remove (view->details->socket_gone_idle_id);
+		g_source_remove (view->details->socket_gone_idle_id);
 		view->details->socket_gone_idle_id = 0;
 	}
 }
@@ -253,9 +273,23 @@ nautilus_view_frame_finalize (GObject *object)
 
 static void
 emit_zoom_parameters_changed (NautilusViewFrame *view)
-{
-	if (view->details->zoomable_frame != NULL) {
+{	
+	eel_g_list_free_deep (view->details->zoom_levels);
+
+	if (view->details->zoomable_frame) {
+		view->details->min_zoom_level = bonobo_zoomable_frame_get_min_zoom_level (view->details->zoomable_frame);
+		view->details->max_zoom_level = bonobo_zoomable_frame_get_max_zoom_level (view->details->zoomable_frame);
+		view->details->has_min_zoom_level = bonobo_zoomable_frame_has_min_zoom_level (view->details->zoomable_frame);
+		view->details->has_max_zoom_level = bonobo_zoomable_frame_has_max_zoom_level (view->details->zoomable_frame);
+		view->details->zoom_levels = bonobo_zoomable_frame_get_preferred_zoom_levels (view->details->zoomable_frame);
+		
 		g_signal_emit (view, signals[ZOOM_PARAMETERS_CHANGED], 0);
+	} else {
+		view->details->min_zoom_level = 0.0;
+		view->details->max_zoom_level = 0.0;
+		view->details->has_min_zoom_level = FALSE;
+		view->details->has_max_zoom_level = FALSE;
+		view->details->zoom_levels = NULL;
 	}
 }
 
@@ -337,7 +371,6 @@ view_frame_underway (NautilusViewFrame *view)
 
 /* stimulus 
    - open_location call from component
-   - open_location_in_new_window
    - report_selection_change
    - report_status
    - set_title 
@@ -420,7 +453,8 @@ view_frame_failed (NautilusViewFrame *view)
 
 NautilusViewFrame *
 nautilus_view_frame_new (BonoboUIContainer *ui_container,
-                         NautilusUndoManager *undo_manager)
+                         NautilusUndoManager *undo_manager,
+			 Nautilus_WindowType window_type)
 {
 	NautilusViewFrame *view_frame;
 	
@@ -429,6 +463,7 @@ nautilus_view_frame_new (BonoboUIContainer *ui_container,
 	bonobo_object_ref (ui_container);
 	view_frame->details->ui_container = ui_container;
 	view_frame->details->undo_manager = undo_manager;
+	view_frame->details->window_type = window_type;
 	
 	return view_frame;
 }
@@ -437,6 +472,8 @@ static void
 emit_zoom_parameters_changed_callback (gpointer data,
 				       gpointer callback_data)
 {
+	
+
 	emit_zoom_parameters_changed (NAUTILUS_VIEW_FRAME (data));
 }
 
@@ -457,7 +494,13 @@ static void
 emit_zoom_level_changed_callback (gpointer data,
 				  gpointer callback_data)
 {
-	g_signal_emit (data,
+	NautilusViewFrame *view;
+	
+	view = NAUTILUS_VIEW_FRAME (data);
+	
+	view->details->zoom_level = bonobo_zoomable_frame_get_zoom_level (view->details->zoomable_frame);
+
+	g_signal_emit (view,
 			 signals[ZOOM_LEVEL_CHANGED], 0,
 			 * (float *) callback_data);
 }
@@ -483,7 +526,8 @@ zoom_level_changed_callback (BonoboZoomableFrame *zframe,
 enum {
 	BONOBO_PROPERTY_TITLE,
 	BONOBO_PROPERTY_HISTORY,
-	BONOBO_PROPERTY_SELECTION
+	BONOBO_PROPERTY_SELECTION,
+	BONOBO_PROPERTY_WINDOW_TYPE,
 };
 
 static Nautilus_History *
@@ -523,6 +567,12 @@ nautilus_view_frame_get_prop (BonoboPropertyBag *bag,
 		g_warning ("NautilusViewFrame: selection fetch not yet implemented");
 		break;
 
+	case BONOBO_PROPERTY_WINDOW_TYPE :
+		BONOBO_ARG_SET_GENERAL (arg, view->details->window_type,
+					TC_Nautilus_WindowType,
+					Nautilus_WindowType,
+					NULL);
+		break;
 	default:
 		g_warning ("NautilusViewFrame: Unknown property idx %d", arg_id);
 		break;
@@ -563,6 +613,15 @@ create_ambient_properties (NautilusViewFrame *view)
 		 _("the current selection"),
 		 BONOBO_PROPERTY_READABLE);
 
+	bonobo_property_bag_add
+		(bag,
+		 "window-type",
+		 BONOBO_PROPERTY_WINDOW_TYPE,
+		 TC_Nautilus_WindowType,
+		 NULL,
+		 _("the type of window the view is embedded in"),
+		 BONOBO_PROPERTY_READABLE);
+
 	view->details->event_source = bag->es;
 
 	return bag;
@@ -574,6 +633,7 @@ create_corba_objects (NautilusViewFrame *view)
 	CORBA_Environment ev;
 	Bonobo_Control control;
 	Bonobo_Zoomable zoomable;
+	Nautilus_ScrollPositionable positionable;
 	BonoboPropertyBag *bag;
 
 	CORBA_exception_init (&ev);
@@ -611,6 +671,18 @@ create_corba_objects (NautilusViewFrame *view)
 
 	CORBA_exception_free (&ev);
 
+	CORBA_exception_init (&ev);
+	
+	positionable = Bonobo_Unknown_queryInterface
+		(view->details->view, "IDL:Nautilus/ScrollPositionable:1.0", &ev);
+	if (!BONOBO_EX (&ev)
+	    && !CORBA_Object_is_nil (positionable, &ev)
+	    && !BONOBO_EX (&ev)) {
+		view->details->positionable = positionable;
+	}
+
+	CORBA_exception_free (&ev);
+	
 	/* Aggregate all the interfaces into one object. */
 	bonobo_object_add_interface (BONOBO_OBJECT (view->details->view_frame), 
 	                             BONOBO_OBJECT (view->details->control_frame));
@@ -641,7 +713,7 @@ queue_view_frame_failed (NautilusViewFrame *view)
 
 	if (view->details->failed_idle_id == 0) {
 		view->details->failed_idle_id =
-			gtk_idle_add (view_frame_failed_callback, view);
+			g_idle_add (view_frame_failed_callback, view);
 	}
 }
 
@@ -695,7 +767,7 @@ check_socket_gone_callback (GtkContainer *container,
 	 * widget in there.
 	 */
 	if (frame->details->socket_gone_idle_id == 0) {
-		frame->details->socket_gone_idle_id = gtk_idle_add
+		frame->details->socket_gone_idle_id = g_idle_add
 			(check_socket_gone_idle_callback, callback_data);
 	}
 }
@@ -750,6 +822,14 @@ attach_view (NautilusViewFrame *view,
 	gtk_container_add (GTK_CONTAINER (view), widget);
 	
 	view_frame_activated (view);
+
+	/* The frame might already be mapped when the view is activated.
+	 * This means we won't get the mapped signal while its active, so
+	 * activate it now.
+	 */
+	if (GTK_WIDGET_MAPPED (GTK_WIDGET (view))) {
+		bonobo_control_frame_control_activate (view->details->control_frame);
+	}
 }
 
 static void
@@ -854,6 +934,12 @@ nautilus_view_frame_selection_changed (NautilusViewFrame *view,
 		return;
 	}
 
+	if (!bonobo_event_source_has_listener
+	    (view->details->event_source,
+	     "Bonobo/Property:change:selection")) {
+		return;
+	}
+
 	uri_list = nautilus_uri_list_from_g_list (selection);
 	
 	CORBA_exception_init (&ev);
@@ -911,7 +997,7 @@ nautilus_view_frame_get_is_zoomable (NautilusViewFrame *view)
 	return view->details->zoomable_frame != NULL;
 }
 
-double
+float
 nautilus_view_frame_get_zoom_level (NautilusViewFrame *view)
 {
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
@@ -920,12 +1006,12 @@ nautilus_view_frame_get_zoom_level (NautilusViewFrame *view)
 		return 0.0;
 	}
 
-	return bonobo_zoomable_frame_get_zoom_level (view->details->zoomable_frame);
+	return view->details->zoom_level;
 }
 
 void
 nautilus_view_frame_set_zoom_level (NautilusViewFrame *view,
-				    double zoom_level)
+				    float zoom_level)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
@@ -933,10 +1019,10 @@ nautilus_view_frame_set_zoom_level (NautilusViewFrame *view,
 		return;
 	}
 
-	bonobo_zoomable_frame_set_zoom_level (view->details->zoomable_frame, (float) zoom_level);
+	bonobo_zoomable_frame_set_zoom_level (view->details->zoomable_frame, zoom_level);
 }
 
-double
+float
 nautilus_view_frame_get_min_zoom_level (NautilusViewFrame *view)
 {
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
@@ -945,10 +1031,10 @@ nautilus_view_frame_get_min_zoom_level (NautilusViewFrame *view)
 		return 0.0;
 	}
 
-	return bonobo_zoomable_frame_get_min_zoom_level (view->details->zoomable_frame);
+	return view->details->min_zoom_level;
 }
 
-double
+float
 nautilus_view_frame_get_max_zoom_level (NautilusViewFrame *view)
 {
 	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), 0.0);
@@ -957,7 +1043,7 @@ nautilus_view_frame_get_max_zoom_level (NautilusViewFrame *view)
 		return 0.0;
 	}
 
-	return bonobo_zoomable_frame_get_max_zoom_level (view->details->zoomable_frame);
+	return view->details->max_zoom_level;
 }
 
 gboolean
@@ -969,7 +1055,7 @@ nautilus_view_frame_get_has_min_zoom_level (NautilusViewFrame *view)
 		return FALSE;
 	}
 
-	return bonobo_zoomable_frame_has_min_zoom_level (view->details->zoomable_frame);
+	return view->details->has_min_zoom_level;
 }
 
 gboolean
@@ -981,7 +1067,7 @@ nautilus_view_frame_get_has_max_zoom_level (NautilusViewFrame *view)
 		return FALSE;
 	}
 
-	return bonobo_zoomable_frame_has_max_zoom_level (view->details->zoomable_frame);
+	return view->details->has_max_zoom_level;
 }
 
 gboolean
@@ -996,6 +1082,24 @@ nautilus_view_frame_get_is_continuous (NautilusViewFrame *view)
 	return bonobo_zoomable_frame_is_continuous (view->details->zoomable_frame);
 }
 
+gboolean
+nautilus_view_frame_get_can_zoom_in (NautilusViewFrame *view)
+{       
+	return !view->details->has_max_zoom_level ||
+                (view->details->zoom_level
+                 < view->details->max_zoom_level);
+
+}
+
+gboolean
+nautilus_view_frame_get_can_zoom_out (NautilusViewFrame *view)
+{       
+	return !view->details->has_min_zoom_level ||
+                (view->details->zoom_level
+                 > view->details->min_zoom_level);
+}
+
+
 GList *
 nautilus_view_frame_get_preferred_zoom_levels (NautilusViewFrame *view)
 {
@@ -1044,6 +1148,47 @@ nautilus_view_frame_zoom_to_fit (NautilusViewFrame *view)
 	bonobo_zoomable_frame_zoom_to_fit (view->details->zoomable_frame);
 }
 
+char *
+nautilus_view_frame_get_first_visible_file (NautilusViewFrame *view)
+{
+	Nautilus_URI uri;
+	char *ret;
+
+	g_return_val_if_fail (NAUTILUS_IS_VIEW_FRAME (view), NULL);
+
+	ret = NULL;
+	if (view->details->positionable) {
+		CORBA_Environment ev;
+		
+		CORBA_exception_init (&ev);
+		uri = Nautilus_ScrollPositionable_get_first_visible_file (view->details->positionable, &ev);
+		ret = g_strdup (uri);
+		CORBA_free (uri);
+		CORBA_exception_free (&ev);
+	}
+
+	return ret;
+}
+
+void
+nautilus_view_frame_scroll_to_file (NautilusViewFrame *view,
+				    const char        *uri)
+{
+	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
+
+	if (view->details->positionable) {
+		CORBA_Environment ev;
+		
+		CORBA_exception_init (&ev);
+
+		Nautilus_ScrollPositionable_scroll_to_file (view->details->positionable,
+							    uri,
+							    &ev);
+		CORBA_exception_free (&ev);
+	}
+}
+
+
 const char *
 nautilus_view_frame_get_view_iid (NautilusViewFrame *view)
 {
@@ -1053,37 +1198,11 @@ nautilus_view_frame_get_view_iid (NautilusViewFrame *view)
 }
 
 void
-nautilus_view_frame_open_location_in_this_window (NautilusViewFrame *view,
-						  const char *location)
-{
-	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
-
-	if (view->details->state == VIEW_FRAME_FAILED) {
-		return;
-	}
-
-	view_frame_wait_is_over (view);
-	g_signal_emit (view, signals[OPEN_LOCATION_IN_THIS_WINDOW], 0, location);
-}
-
-void
-nautilus_view_frame_open_location_prefer_existing_window (NautilusViewFrame *view,
-							  const char *location)
-{
-	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
-
-	if (view->details->state == VIEW_FRAME_FAILED) {
-		return;
-	}
-
-	view_frame_wait_is_over (view);
-	g_signal_emit (view, signals[OPEN_LOCATION_PREFER_EXISTING_WINDOW], 0, location);
-}
-
-void
-nautilus_view_frame_open_location_force_new_window (NautilusViewFrame *view,
-						    const char *location,
-						    GList *selection)
+nautilus_view_frame_open_location (NautilusViewFrame *view,
+				   const char *location,
+				   Nautilus_ViewFrame_OpenMode mode,
+				   Nautilus_ViewFrame_OpenFlags flags,
+				   GList *selection)
 {
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (view));
 
@@ -1093,8 +1212,8 @@ nautilus_view_frame_open_location_force_new_window (NautilusViewFrame *view,
 
 	view_frame_wait_is_over (view);
 	g_signal_emit (view,
-		       signals[OPEN_LOCATION_FORCE_NEW_WINDOW], 0,
-		       location, selection);
+		       signals[OPEN_LOCATION], 0,
+		       location, mode, flags, selection);
 }
 
 void
@@ -1319,6 +1438,12 @@ send_history (NautilusViewFrame *view)
 		return;
 	}
 
+	if (!bonobo_event_source_has_listener
+	    (view->details->event_source,
+	     "Bonobo/Property:change:history")) {
+		return;
+	}
+
 	history = get_history_list (view);
 	if (history == CORBA_OBJECT_NIL) {
 		return;
@@ -1452,33 +1577,15 @@ nautilus_view_frame_class_init (NautilusViewFrameClass *class)
 		 NULL, NULL,
 		 g_cclosure_marshal_VOID__VOID,
 		 G_TYPE_NONE, 0);
-	signals[OPEN_LOCATION_FORCE_NEW_WINDOW] = g_signal_new
-		("open_location_force_new_window",
+	signals[OPEN_LOCATION] = g_signal_new
+		("open_location",
 		 G_TYPE_FROM_CLASS (class),
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (NautilusViewFrameClass, 
-				  open_location_force_new_window),
+				  open_location),
 		 NULL, NULL,
-		 eel_marshal_VOID__STRING_POINTER,
-		 G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
-	signals[OPEN_LOCATION_IN_THIS_WINDOW] = g_signal_new
-		("open_location_in_this_window",
-		 G_TYPE_FROM_CLASS (class),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET (NautilusViewFrameClass, 
-				  open_location_in_this_window),
-		 NULL, NULL,
-		 g_cclosure_marshal_VOID__STRING,
-		 G_TYPE_NONE, 1, G_TYPE_STRING);
-	signals[OPEN_LOCATION_PREFER_EXISTING_WINDOW] = g_signal_new
-		("open_location_prefer_existing_window",
-		 G_TYPE_FROM_CLASS (class),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET (NautilusViewFrameClass, 
-				  open_location_in_this_window),
-		 NULL, NULL,
-		 g_cclosure_marshal_VOID__STRING,
-		 G_TYPE_NONE, 1, G_TYPE_STRING);
+		 eel_marshal_VOID__STRING_LONG_LONG_POINTER,
+		 G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_LONG, G_TYPE_LONG, G_TYPE_POINTER);
 	signals[REPORT_LOCATION_CHANGE] = g_signal_new
 		("report_location_change",
 		 G_TYPE_FROM_CLASS (class),
